@@ -1,5 +1,5 @@
 <?php
-require_once("../inc/always.php");
+require_once("./always.php");
 dbg_error_log( "freebusy", " User agent: %s", ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "Unfortunately Mulberry and Chandler don't send a 'User-agent' header with their requests :-(")) );
 dbg_log_array( "headers", '_SERVER', $_SERVER, true );
 if ( isset($c->public_freebusy_url) && $c->public_freebusy_url ) {
@@ -42,7 +42,7 @@ if ( isset($fb_format) && $fb_format != 'text/calendar' ) {
 */
 $request = new CalDAVRequest(array("allow_by_email" => 1));
 
-if ( ! $request->AllowedTo('freebusy') ) $request->DoResponse( 404 );
+if ( ! $request->HavePrivilegeTo('read-free-busy') ) $request->DoResponse( 404 );
 
 switch ( $_SERVER['REQUEST_METHOD'] ) {
   case 'GET':
