@@ -9,7 +9,7 @@
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
 */
 
-require_once("../inc/always.php");
+require_once("./always.php");
 require_once("DAViCalSession.php");
 $session->LoginRequired();
 
@@ -24,15 +24,15 @@ if( function_exists("sync_LDAP") && isset($_POST['Sync_LDAP'])){
   sync_LDAP();
 }
 
+if( function_exists("sync_LDAP_groups") && isset($_POST['Sync_LDAP_groups'])){
+  sync_LDAP_groups();
+}
+
 if(isset($_POST['import_from_directory'])){
   Tools::importFromDirectory();
 }
 
 
-include("page-header.php");
-$Tools = new Tools();
-
-$Tools->render();
 class Tools {
 
   function render(){
@@ -52,15 +52,19 @@ class Tools {
     $html .= "<table width=\"100%\" class=\"data\">\n";
     $html .= $ef->StartForm( array("autocomplete" => "off" ) );
     $html .= sprintf( "<tr><td style=\"text-align:left\" colspan=\"2\" >%s</td></tr>\n",
-    translate("This operation does the following: <ul><li>check valid users in LDAP directory</li>
-    <li>check users in RSCDS</li></ul> then
-    <ul><li>if a user is present in RSCDS but not in LDAP set him as inactive in RSCDS</li>
-        <li>if a user is present in LDAP but not in RSCDS create the user in RSCDS</li>
-        <li>if a user in present in LDAP and RSCDS then update information in RSCDS</li>
-    </ul>"));
+    translate("This operation does the following: <ul><li>check valid users in LDAP directory</li> <li>check users in DAViCal</li></ul> then <ul><li>if a user is present in DAViCal but not in LDAP set him as inactive in DAViCal</li> <li>if a user is present in LDAP but not in DAViCal create the user in DAViCal</li> <li>if a user in present in LDAP and DAViCal then update information in DAViCal</li> </ul>"));
     $html .= "</table>\n";
 
     $html .= $ef->SubmitButton( "Sync_LDAP", translate('Submit'));
+		
+    $html .= '<h1>'.translate('Sync LDAP Groups with RSCDS') .'</h1>';
+    $html .= "<table width=\"100%\" class=\"data\">\n";
+    $html .= $ef->StartForm( array("autocomplete" => "off" ) );
+    $html .= sprintf( "<tr><td style=\"text-align:left\" colspan=\"2\" >%s</td></tr>\n",
+    translate("This operation does the following: <ul><li>check valid groups in LDAP directory</li> <li>check groups in DAViCal</li></ul> then <ul><li>if a group is present in DAViCal but not in LDAP set as inactive in DAViCal</li> <li>if a group is present in LDAP but not in DAViCal create the group in DAViCal</li> <li>if a group in present in LDAP and DAViCal then update information in DAViCal</li> </ul>"));
+    $html .= "</table>\n";
+		
+    $html .= $ef->SubmitButton( "Sync_LDAP_groups", translate('Submit'));
     $html .= $ef->EndForm();
 
     $html .= "</div>";
@@ -110,16 +114,14 @@ class Tools {
     }
     $dir = $_POST["directory_path"];
     if(!is_readable($dir)){
-      $c->messages[] = sprintf(i18n('directory %s is not readable'),htmlentities($dir));
+      $c->messages[] = sprintf(i18n('directory %s is not readable'),htmlspecialchars($dir));
       dbg_error_log( "importFromDirectory", "directory is not readable");
       return ;
     }
     if ($handle = opendir($dir)) {
       while (false !== ($file = readdir($handle))) {
-        if ($file != "." && $file != ".." && substr($file,-4) != '.ics') {
-          continue;
-        }
-        if(!is_readable($dir.'/'.$file)){
+        if ($file == "." || $file == ".." || substr($file,-4) != '.ics') continue;
+        if ( !is_readable($dir.'/'.$file) ) {
           dbg_error_log( "importFromDirectory", "ics file '%s' is not readable",$dir .'/'.$file);
           continue;
         }
@@ -132,17 +134,18 @@ class Tools {
           if ( check_string($ics) ) {
             $path = "/".substr($file,0,-4).$path_ics;
             dbg_error_log( "importFromDirectory", "importing to $path");
-            include_once("caldav-PUT-functions.php");
+            $c->readonly_webdav_collections = false;  // Override this setting so we can create collections/events on import.
+            require_once("caldav-PUT-functions.php");
             if ( $user = getUserByName(substr($file,0,-4),'importFromDirectory',__LINE__,__FILE__)) {
               $user_no = $user->user_no;
             }
             if(controlRequestContainer(substr($file,0,-4),$user_no, $path,false) === -1)
               continue;
             import_collection($ics,$user_no,$path,1);
-            $c->messages[] = sprintf(translate("all events of user %s were deleted and replaced by those from file %s"),substr($file,0,-4),$dir.'/'.$file);
+            $c->messages[] = sprintf(translate('all events of user %s were deleted and replaced by those from file %s'),substr($file,0,-4),$dir.'/'.$file);
           }
           else {
-            $c->messages[] =  sprintf(translate("the file %s is not UTF-8 encoded, please check error for more details"),$dir.'/'.$file);
+            $c->messages[] = sprintf(translate('the file %s is not UTF-8 encoded, please check error for more details'),$dir.'/'.$file);
           }
         }
       }
@@ -151,5 +154,8 @@ class Tools {
   }
 }
 
-include("page-footer.php");
+$Tools = new Tools();
 
+include("page-header.php");
+$Tools->render();
+include("page-footer.php");
