@@ -10,12 +10,13 @@
 * @package   davical
 * @subpackage   Request
 * @author    Andrew McMillan <andrew@mcmillan.net.nz>
-* @copyright Catalyst .Net Ltd
-* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
+* @copyright Catalyst .Net Ltd, Morphoss Ltd
+* @license   http://gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
 
-require_once("XMLElement.php");
+require_once("XMLDocument.php");
 require_once("CalDAVPrincipal.php");
+include("DAVTicket.php");
 
 define('DEPTH_INFINITY', 9999);
 
@@ -77,30 +78,59 @@ class CalDAVRequest
   */
   var $collection_type;
 
+  /**
+  * The type of collection being requested:
+  *  calendar, schedule-inbox, schedule-outbox
+  */
+  protected $exists;
+
+  /**
+  * The decimal privileges allowed by this user to the identified resource.
+  */
+  protected $privileges;
+
   /**
   * A static structure of supported privileges.
   */
   var $supported_privileges;
 
+  /**
+  * A DAVTicket object, if there is a ?ticket=id or Ticket: id with this request
+  */
+  public $ticket;
+
   /**
   * Create a new CalDAVRequest object.
   */
-  function CalDAVRequest( $options = array() ) {
+  function __construct( $options = array() ) {
     global $session, $c, $debugging;
 
     $this->supported_privileges = array(
       'all' => array(
-        'read' => 'Read the content of a resource or collection',
+        'read' => translate('Read the content of a resource or collection'),
         'write' => array(
-          'bind' => 'Create a resource or collection',
-          'unbind' => 'Delete a resource or collection',
-          'write-content' => 'Write content',
-          'write-properties' => 'Write properties'
+          'bind' => translate('Create a resource or collection'),
+          'unbind' => translate('Delete a resource or collection'),
+          'write-content' => translate('Write content'),
+          'write-properties' => translate('Write properties')
         ),
-        'urn:ietf:params:xml:ns:caldav:read-free-busy' => 'Read the free/busy information for a calendar collection',
-        'read-acl' => 'Read ACLs for a resource or collection',
-        'write-acl' => 'Write ACLs for a resource or collection',
-        'unlock' => 'Remove a lock'
+        'urn:ietf:params:xml:ns:caldav:read-free-busy' => translate('Read the free/busy information for a calendar collection'),
+        'read-acl' => translate('Read ACLs for a resource or collection'),
+        'read-current-user-privilege-set' => translate('Read the details of the current user\'s access control to this resource.'),
+        'write-acl' => translate('Write ACLs for a resource or collection'),
+        'unlock' => translate('Remove a lock'),
+
+        'urn:ietf:params:xml:ns:caldav:schedule-deliver' => array(
+          'urn:ietf:params:xml:ns:caldav:schedule-deliver-invite'=> translate('Deliver scheduling invitations from an organiser to this scheduling inbox'),
+          'urn:ietf:params:xml:ns:caldav:schedule-deliver-reply' => translate('Deliver scheduling replies from an attendee to this scheduling inbox'),
+          'urn:ietf:params:xml:ns:caldav:schedule-query-freebusy' => translate('Allow free/busy enquiries targeted at the owner of this scheduling inbox')
+        ),
+
+        'urn:ietf:params:xml:ns:caldav:schedule-send' => array(
+          'urn:ietf:params:xml:ns:caldav:schedule-send-invite' => translate('Send scheduling invitations as an organiser from the owner of this scheduling outbox.'),
+          'urn:ietf:params:xml:ns:caldav:schedule-send-reply' => translate('Send scheduling replies as an attendee from the owner of this scheduling outbox.'),
+          'urn:ietf:params:xml:ns:caldav:schedule-send-freebusy' => translate('Send free/busy enquiries')
+        )
       )
     );
 
@@ -129,8 +159,14 @@ class CalDAVRequest
     if ( isset($debugging) && isset($_GET['method']) ) {
       $_SERVER['REQUEST_METHOD'] = $_GET['method'];
     }
+    else if ( $_SERVER['REQUEST_METHOD'] == 'POST' && isset($_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE']) ){
+      $_SERVER['REQUEST_METHOD'] = $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'];
+    }
     $this->method = $_SERVER['REQUEST_METHOD'];
-
+    $this->content_type = (isset($_SERVER['CONTENT_TYPE']) ? $_SERVER['CONTENT_TYPE'] : null);
+    if ( preg_match( '{^(\S+/\S+)\s*(;.*)?$}', $this->content_type, $matches ) ) {
+      $this->content_type = $matches[1];
+    }
     $this->user_agent = ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "Probably Mulberry"));
 
     /**
@@ -166,7 +202,7 @@ class CalDAVRequest
     * MOVE/COPY use a "Destination" header and (optionally) an "Overwrite" one.
     */
     if ( isset($_SERVER['HTTP_DESTINATION']) ) $this->destination = $_SERVER['HTTP_DESTINATION'];
-    $this->overwrite = ( isset($_SERVER['HTTP_OVERWRITE']) ? $_SERVER['HTTP_OVERWRITE'] : 'T' ); // RFC2518, 9.6 says default True.
+    $this->overwrite = ( isset($_SERVER['HTTP_OVERWRITE']) && ($_SERVER['HTTP_OVERWRITE'] == 'F') ? false : true ); // RFC4918, 9.8.4 says default True.
 
     /**
     * LOCK things use an "If" header to hold the lock in some cases, and "Lock-token" in others
@@ -176,6 +212,16 @@ class CalDAVRequest
       $this->lock_token = $matches[1];
     }
 
+    /**
+    * Check for an access ticket.
+    */
+    if ( isset($_GET['ticket']) ) {
+      $this->ticket = new DAVTicket($_GET['ticket']);
+    }
+    else if ( isset($_SERVER['HTTP_TICKET']) ) {
+      $this->ticket = new DAVTicket($_SERVER['HTTP_TICKET']);
+    }
+
     /**
     * LOCK things use a "Timeout" header to set a series of reducing alternative values
     */
@@ -242,14 +288,16 @@ class CalDAVRequest
     * The collection URL for this request is therefore the longest row in the result, so we
     * can "... ORDER BY LENGTH(dav_name) DESC LIMIT 1"
     */
-    $sql = "SELECT * FROM collection WHERE dav_name = ".qpg($this->path);
+    $sql = "SELECT * FROM collection WHERE dav_name = :exact_name";
+    $params = array( ':exact_name' => $this->path );
     if ( !preg_match( '#/$#', $this->path ) ) {
-      $sql .= " OR dav_name = ".qpg(preg_replace( '#[^/]*$#', '', $this->path));
-      $sql .= " OR dav_name = ".qpg($this->path."/");
+      $sql .= " OR dav_name = :truncated_name OR dav_name = :trailing_slash_name";
+      $params[':truncated_name'] = preg_replace( '#[^/]*$#', '', $this->path);
+      $params[':trailing_slash_name'] = $this->path."/";
     }
-    $sql .= "ORDER BY LENGTH(dav_name) DESC LIMIT 1";
-    $qry = new PgQuery( $sql );
-    if ( $qry->Exec('caldav') && $qry->rows == 1 && ($row = $qry->Fetch()) ) {
+    $sql .= " ORDER BY LENGTH(dav_name) DESC LIMIT 1";
+    $qry = new AwlQuery( $sql, $params );
+    if ( $qry->Exec('caldav',__LINE__,__FILE__) && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
       if ( $row->dav_name == $this->path."/" ) {
         $this->path = $row->dav_name;
         dbg_error_log( "caldav", "Path is actually a collection - sending Content-Location header." );
@@ -260,26 +308,35 @@ class CalDAVRequest
       $this->collection_path = $row->dav_name;
       $this->collection_type = ($row->is_calendar == 't' ? 'calendar' : 'collection');
       $this->collection = $row;
-      $this->collection->type = $this->collection_type; 
+      if ( preg_match( '#^((/[^/]+/)\.(in|out)/)[^/]*$#', $this->path, $matches ) ) {
+        $this->collection_type = 'schedule-'. $matches[3]. 'box';
+      }
+      $this->collection->type = $this->collection_type;
     }
-    else if ( preg_match( '#^((/[^/]+/)\.(in|out)/)[^/]*$#', $this->path, $matches ) ) {
+    else if ( preg_match( '{^( ( / ([^/]+) / ) \.(in|out)/ ) [^/]*$}x', $this->path, $matches ) ) {
       // The request is for a scheduling inbox or outbox (or something inside one) and we should auto-create it
-      $displayname = $session->fullname . ($matches[3] == 'in' ? ' Inbox' : ' Outbox');
-      $this->collection_type = 'schedule-'. $matches[3]. 'box';
+      $params = array( ':username' => $matches[3], ':parent_container' => $matches[2], ':dav_name' => $matches[1] );
+      $params[':boxname'] = ($matches[4] == 'in' ? ' Inbox' : ' Outbox');
+      $this->collection_type = 'schedule-'. $matches[4]. 'box';
+      $params[':resourcetypes'] = sprintf('<DAV::collection/><urn:ietf:params:xml:ns:caldav:%s/>', $this->collection_type );
       $sql = <<<EOSQL
-INSERT INTO collection ( user_no, parent_container, dav_name, dav_displayname, is_calendar, created, modified, dav_etag )
-    VALUES( ?, ?, ?, ?, FALSE, current_timestamp, current_timestamp, '1' )
+INSERT INTO collection ( user_no, parent_container, dav_name, dav_displayname, is_calendar, created, modified, dav_etag, resourcetypes )
+    VALUES( (SELECT user_no FROM usr WHERE username = :username),
+            :parent_container, :dav_name,
+            (SELECT fullname FROM usr WHERE username = :username) || :boxname,
+             FALSE, current_timestamp, current_timestamp, '1', :resourcetypes )
 EOSQL;
-      $qry = new PgQuery( $sql, $session->user_no, $matches[2] , $matches[1], $displayname );
-      $qry->Exec('caldav');
-      dbg_error_log( "caldav", "Created new collection as '$displayname'." );
 
-      $qry = new PgQuery( "SELECT * FROM collection WHERE user_no = ? AND dav_name = ?;", $session->user_no, $matches[1] );
-      if ( $qry->Exec('caldav') && $qry->rows == 1 && ($row = $qry->Fetch()) ) {
+      $qry = new AwlQuery( $sql, $params );
+      $qry->Exec('caldav',__LINE__,__FILE__);
+      dbg_error_log( 'caldav', 'Created new collection as "%s".', trim($params[':boxname']) );
+
+      $qry = new AwlQuery( "SELECT * FROM collection WHERE dav_name = :dav_name", array( ':dav_name' => $matches[1] ) );
+      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
         $this->collection_id = $row->collection_id;
         $this->collection_path = $matches[1];
         $this->collection = $row;
-        $this->collection->type = $this->collection_type; 
+        $this->collection->type = $this->collection_type;
       }
     }
     else if ( preg_match( '#^((/[^/]+/)calendar-proxy-(read|write))/?[^/]*$#', $this->path, $matches ) ) {
@@ -299,6 +356,7 @@ EOSQL;
       $this->collection_type = 'email';
       $this->collection_path = $this->path;
       $this->_is_principal = true;
+//      $this->by_email = true;
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->path, $matches) || preg_match( '#^(/principals/[^/]+/[^/]+)/?$#', $this->path, $matches) ) {
       $this->collection_id = -1;
@@ -330,10 +388,18 @@ EOSQL;
     $this->principal = new CalDAVPrincipal( array( "path" => $this->path, "options" => $this->options ) );
     if ( isset($this->principal->user_no) ) $this->user_no  = $this->principal->user_no;
     if ( isset($this->principal->username)) $this->username = $this->principal->username;
-    if ( isset($this->principal->by_email)) $this->by_email = true;
+    if ( isset($this->principal->by_email) && $this->principal->by_email) $this->by_email = true;
+    if ( isset($this->principal->principal_id)) $this->principal_id = $this->principal->principal_id;
 
-    if ( $this->collection_type == 'principal' || $this->collection_type == 'email' ) {
+    if ( $this->collection_type == 'principal' || $this->collection_type == 'email' || $this->collection_type == 'proxy' ) {
       $this->collection = $this->principal->AsCollection();
+      if( $this->collection_type == 'proxy' ) {
+        $this->collection = $this->principal->AsCollection();
+        $this->collection->is_proxy = 't';
+        $this->collection->type = 'proxy';
+        $this->collection->proxy_type = $this->proxy_type;
+        $this->collection->dav_displayname = sprintf('Proxy %s for %s', $this->proxy_type, $this->principal->username() );
+      }
     }
     elseif( $this->collection_type == 'root' ) {
       $this->collection = (object) array(
@@ -348,41 +414,111 @@ EOSQL;
                             'created' => date('Ymd\THis')
                           );
     }
-    elseif( $this->collection_type == 'proxy' ) {
-      $this->collection = (object) array(
-          'dav_name' => $this->collection_path,
-          'is_calendar' => 'f',
-          'is_principal' => 't',
-          'is_proxy' => 't',
-          'type' => 'proxy',
-          'proxy_type' => $this->proxy_type,
-          'dav_displayname' => sprintf('Proxy %s for %s', $this->proxy_type, $this->principal->username),
-          'collection_id' => 0,
-          'user_no' => $this->principal->user_no,
-          'username' => $this->principal->username,
-          'email' => $this->principal->email,
-          'created' => $this->principal->created,
-          'dav_etag' => $this->principal->created
-      );
-    }
 
     /**
     * Evaluate our permissions for accessing the target
     */
     $this->setPermissions();
 
+    $this->supported_methods = array(
+      'OPTIONS' => '',
+      'PROPFIND' => '',
+      'REPORT' => '',
+      'DELETE' => '',
+      'LOCK' => '',
+      'UNLOCK' => '',
+      'MOVE' => '',
+      'ACL' => ''
+    );
+    if ( $this->IsCollection() ) {
+      switch ( $this->collection_type ) {
+        case 'root':
+        case 'email':
+          // We just override the list completely here.
+          $this->supported_methods = array(
+            'OPTIONS' => '',
+            'PROPFIND' => '',
+            'REPORT' => ''
+          );
+          break;
+        case 'schedule-inbox':
+        case 'schedule-outbox':
+          $this->supported_methods = array_merge(
+            $this->supported_methods,
+            array(
+              'POST' => '', 'GET' => '', 'PUT' => '', 'HEAD' => '', 'PROPPATCH' => ''
+            )
+          );
+          break;
+        case 'calendar':
+          $this->supported_methods['GET'] = '';
+          $this->supported_methods['PUT'] = '';
+          $this->supported_methods['HEAD'] = '';
+          break;
+        case 'collection':
+        case 'principal':
+          $this->supported_methods['GET'] = '';
+          $this->supported_methods['PUT'] = '';
+          $this->supported_methods['HEAD'] = '';
+          $this->supported_methods['MKCOL'] = '';
+          $this->supported_methods['MKCALENDAR'] = '';
+          $this->supported_methods['PROPPATCH'] = '';
+          $this->supported_methods['BIND'] = '';
+          break;
+      }
+    }
+    else {
+      $this->supported_methods = array_merge(
+        $this->supported_methods,
+        array(
+          'GET' => '',
+          'HEAD' => '',
+          'PUT' => ''
+        )
+      );
+    }
+
+    $this->supported_reports = array(
+      'DAV::principal-property-search' => '',
+      'DAV::expand-property' => '',
+      'DAV::sync-collection' => ''
+    );
+    if ( isset($this->collection) && $this->collection->is_calendar ) {
+      $this->supported_reports = array_merge(
+        $this->supported_reports,
+        array(
+          'urn:ietf:params:xml:ns:caldav:calendar-query' => '',
+          'urn:ietf:params:xml:ns:caldav:calendar-multiget' => '',
+          'urn:ietf:params:xml:ns:caldav:free-busy-query' => ''
+        )
+      );
+    }
+
+
     /**
     * If the content we are receiving is XML then we parse it here.  RFC2518 says we
     * should reasonably expect to see either text/xml or application/xml
     */
-    if ( isset($_SERVER['CONTENT_TYPE']) && preg_match( '#(application|text)/xml#', $_SERVER['CONTENT_TYPE'] ) ) {
+    if ( isset($this->content_type) && preg_match( '#(application|text)/xml#', $this->content_type ) ) {
       $xml_parser = xml_parser_create_ns('UTF-8');
       $this->xml_tags = array();
       xml_parser_set_option ( $xml_parser, XML_OPTION_SKIP_WHITE, 1 );
       xml_parser_set_option ( $xml_parser, XML_OPTION_CASE_FOLDING, 0 );
-      xml_parse_into_struct( $xml_parser, $this->raw_post, $this->xml_tags );
+      $rc = xml_parse_into_struct( $xml_parser, $this->raw_post, $this->xml_tags );
+      if ( $rc == false ) {
+        dbg_error_log( 'ERROR', 'XML parsing error: %s at line %d, column %d',
+                    xml_error_string(xml_get_error_code($xml_parser)),
+                    xml_get_current_line_number($xml_parser), xml_get_current_column_number($xml_parser) );
+        $this->XMLResponse( 400, new XMLElement( 'error', new XMLElement('invalid-xml'), array( 'xmlns' => 'DAV:') ) );
+      }
       xml_parser_free($xml_parser);
-      dbg_error_log( "caldav", " Parsed incoming XML request body." );
+      if ( count($this->xml_tags) ) {
+        dbg_error_log( "caldav", " Parsed incoming XML request body." );
+      }
+      else {
+        $this->xml_tags = null;
+        dbg_error_log( "ERROR", "Incoming request sent content-type XML with no XML request body." );
+      }
     }
 
     /**
@@ -407,6 +543,7 @@ EOSQL;
 
     $this->user_no = $session->user_no;
     $this->username = $session->username;
+    $this->principal_id = $session->principal_id;
 
     @dbg_error_log( "WARN", "Call to deprecated CalDAVRequest::UserFromPath()" );
 
@@ -421,15 +558,18 @@ EOSQL;
     @dbg_error_log( "caldav", "Path split into at least /// %s /// %s /// %s", $path_split[1], $path_split[2], $path_split[3] );
     if ( isset($this->options['allow_by_email']) && preg_match( '#/(\S+@\S+[.]\S+)/?$#', $this->path, $matches) ) {
       $this->by_email = $matches[1];
-//      $qry = new PgQuery("SELECT user_no FROM usr WHERE email = ? AND get_permissions(?,user_no) ~ '[FRA]';", $this->by_email, $session->user_no );
-      $qry = new PgQuery("SELECT user_no FROM usr WHERE email = ?;", $this->by_email );
-      if ( $qry->Exec("caldav") && $user = $qry->Fetch() ) {
+      $qry = new AwlQuery("SELECT user_no, principal_id, username FROM usr JOIN principal USING (user_no) WHERE email = :email",
+                          array(':email' => $this->by_email ) );
+      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $user = $qry->Fetch() ) {
         $this->user_no = $user->user_no;
+        $this->username = $user->username;
+        $this->principal_id = $user->principal_id;
       }
     }
     elseif( $user = getUserByName($this->username,'caldav',__LINE__,__FILE__)) {
       $this->principal = $user;
       $this->user_no = $user->user_no;
+      $this->principal_id = $user->principal_id;
     }
   }
 
@@ -450,78 +590,60 @@ EOSQL;
     global $c, $session;
 
     if ( $this->path == '/' || $this->path == '' ) {
-      $this->permissions = array("read" => 'read' );
-      dbg_error_log( "caldav", "Read permissions for user accessing /" );
-      return;
+      $this->privileges = privilege_to_bits( array('read','read-free-busy','read-acl'));
+      dbg_error_log( "caldav", "Full read permissions for user accessing /" );
     }
-
-    if ( $session->AllowedTo("Admin") || $session->user_no == $this->user_no ) {
-      $this->permissions = array('all' => 'abstract' );
-      $this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy'] = 'real';
-      $this->permissions['read'] = 'real';
-      $this->permissions['write'] = 'aggregate';
-      $this->permissions['bind'] = 'real';      // PUT of new content (i.e. Create)
-      $this->permissions['unbind'] = 'real';  // DELETE
-      $this->permissions['write-content'] = 'real';        // PUT Modify
-      $this->permissions['write-properties'] = 'real';  // PROPPATCH
-      $this->permissions['lock'] = 'real';
-      $this->permissions['unlock'] = 'real';
-      $this->permissions['read-acl'] = 'real';
-      $this->permissions['read-current-user-privilege-set'] = 'real';
+    else if ( $session->AllowedTo("Admin") || $session->user_no == $this->user_no ) {
+      $this->privileges = privilege_to_bits('all');
       dbg_error_log( "caldav", "Full permissions for %s", ( $session->user_no == $this->user_no ? "user accessing their own hierarchy" : "a systems administrator") );
-      return;
     }
+    else {
+      $this->privileges = 0;
+      if ( $this->IsPublic() ) {
+        $this->privileges = privilege_to_bits(array('read','read-free-busy'));
+        dbg_error_log( "caldav", "Basic read permissions for user accessing a public collection" );
+      }
+      else if ( isset($c->public_freebusy_url) && $c->public_freebusy_url ) {
+        $this->privileges = privilege_to_bits('read-free-busy');
+        dbg_error_log( "caldav", "Basic free/busy permissions for user accessing a public free/busy URL" );
+      }
 
-    $this->permissions = array();
-
-    if ( $this->IsPublic() ) {
-      $this->permissions['read'] = 'real';
-      $this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy'] = 'real';
-    }
-    else if ( isset($c->public_freebusy_url) && $c->public_freebusy_url ) {
-      $this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy'] = 'real';
-    }
-
-    /**
-    * In other cases we need to query the database for permissions
-    */
-    $qry = new PgQuery( "SELECT get_permissions( ?, ? ) AS perm;", $session->user_no, $this->user_no);
-    if ( $qry->Exec("caldav") && $permission_result = $qry->Fetch() ) {
-      $permission_result = "!".$permission_result->perm; // We prepend something to ensure we get a non-zero position.
-      if ( strpos($permission_result,"A") ) {
-        $this->permissions['all'] = 'abstract';
-        $this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy'] = 'real';
-        $this->permissions['read'] = 'real';
-        $this->permissions['write'] = 'aggregate';
-        $this->permissions['bind'] = 'real';      // PUT of new content (i.e. Create)
-        $this->permissions['unbind'] = 'real';  // DELETE
-        $this->permissions['write-content'] = 'real';        // PUT Modify
-        $this->permissions['write-properties'] = 'real';  // PROPPATCH
-        $this->permissions['lock'] = 'real';
-        $this->permissions['unlock'] = 'real';
-        $this->permissions['read-acl'] = 'real';
-        $this->permissions['read-current-user-privilege-set'] = 'real';
+      /**
+      * In other cases we need to query the database for permissions
+      */
+      $params = array( ':session_principal_id' => $session->principal_id, ':scan_depth' => $c->permission_scan_depth );
+      if ( isset($this->by_email) && $this->by_email ) {
+        $sql ='SELECT pprivs( :session_principal_id::int8, :request_principal_id::int8, :scan_depth::int ) AS perm';
+        $params[':request_principal_id'] = $this->principal_id;
       }
       else {
-        if ( strpos($permission_result,"F") )       $this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy'] = 'real';
-        if ( strpos($permission_result,"R") )       $this->permissions['read'] = 'real';
-        if ( strpos($permission_result,"W") ) {
-          $this->permissions['write'] = 'aggregate';
-          $this->permissions['bind'] = 'real';      // PUT of new content (i.e. Create)
-          $this->permissions['unbind'] = 'real';  // DELETE
-          $this->permissions['write-content'] = 'real';        // PUT Modify
-          $this->permissions['write-properties'] = 'real';  // PROPPATCH
-          $this->permissions['lock'] = 'real';
-          $this->permissions['unlock'] = 'real';
-        }
-        else {
-          if ( strpos($permission_result,"C") )       $this->permissions['bind'] = 'real';      // PUT of new content (i.e. Create)
-          if ( strpos($permission_result,"D") )       $this->permissions['unbind'] = 'real';  // DELETE
-          if ( strpos($permission_result,"M") )       $this->permissions['write-content'] = 'real';  // PUT Modify
-        }
+        $sql = 'SELECT path_privs( :session_principal_id::int8, :request_path::text, :scan_depth::int ) AS perm';
+        $params[':request_path'] = $this->path;
+      }
+      $qry = new AwlQuery( $sql, $params );
+      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $permission_result = $qry->Fetch() )
+        $this->privileges |= bindec($permission_result->perm);
+
+      dbg_error_log( 'caldav', 'Restricted permissions for user accessing someone elses hierarchy: %s', decbin($this->privileges) );
+      if ( isset($this->ticket) && $this->ticket->MatchesPath($this->path) ) {
+        $this->privileges |= $this->ticket->privileges();
+        dbg_error_log( 'caldav', 'Applying permissions for ticket "%s" now: %s', $this->ticket->id(), decbin($this->privileges) );
       }
-      dbg_error_log( "caldav", "Restricted permissions for user accessing someone elses hierarchy: %s", implode( ", ", $this->permissions ) );
     }
+
+    /** convert privileges into older style permissions */
+    $this->permissions = array();
+    $privs = bits_to_privilege($this->privileges);
+    foreach( $privs AS $k => $v ) {
+      switch( $v ) {
+        case 'DAV::all':    $type = 'abstract';   break;
+        case 'DAV::write':  $type = 'aggregate';  break;
+        default: $type = 'real';
+      }
+      $v = str_replace('DAV::', '', $v);
+      $this->permissions[$v] = $type;
+    }
+
   }
 
 
@@ -535,12 +657,17 @@ EOSQL;
   function IsLocked() {
     if ( !isset($this->_locks_found) ) {
       $this->_locks_found = array();
+
+      $sql = 'DELETE FROM locks WHERE (start + timeout) < current_timestamp';
+      $qry = new AwlQuery($sql);
+      $qry->Exec('caldav',__LINE__,__FILE__);
+
       /**
       * Find the locks that might apply and load them into an array
       */
-      $sql = "SELECT * FROM locks WHERE ?::text ~ ('^'||dav_name||?)::text;";
-      $qry = new PgQuery($sql, $this->path, ($this->IsInfiniteDepth() ? '' : '$') );
-      if ( $qry->Exec("caldav",__LINE__,__FILE__) ) {
+      $sql = 'SELECT * FROM locks WHERE :dav_name::text ~ (\'^\'||dav_name||:pattern_end_match)::text';
+      $qry = new AwlQuery($sql, array( ':dav_name' => $this->path, ':pattern_end_match' => ($this->IsInfiniteDepth() ? '' : '$') ) );
+      if ( $qry->Exec('caldav',__LINE__,__FILE__) ) {
         while( $lock_row = $qry->Fetch() ) {
           $this->_locks_found[$lock_row->opaquelocktoken] = $lock_row;
         }
@@ -572,6 +699,15 @@ EOSQL;
   }
 
 
+  /**
+  * Returns the dav_name of the resource in our internal namespace
+  */
+  function dav_name() {
+    if ( isset($this->path) ) return $this->path;
+    return null;
+  }
+
+
   /**
   * Returns the name for this depth: 0, 1, infinity
   */
@@ -600,9 +736,8 @@ EOSQL;
       return $this->_locks_found[$lock_token];
     }
 
-    $sql = "SELECT * FROM locks WHERE opaquelocktoken = ?;";
-    $qry = new PgQuery($sql, $lock_token );
-    if ( $qry->Exec("caldav",__LINE__,__FILE__) ) {
+    $qry = new AwlQuery('SELECT * FROM locks WHERE opaquelocktoken = :lock_token', array( ':lock_token' => $lock_token ) );
+    if ( $qry->Exec('caldav',__LINE__,__FILE__) ) {
       $lock_row = $qry->Fetch();
       $this->_locks_found = array( $lock_token => $lock_row );
       return $this->_locks_found[$lock_token];
@@ -713,8 +848,17 @@ EOSQL;
   * Returns true if the URL referenced by this request points at a calendar collection.
   */
   function IsCalendar( ) {
-    if ( !$this->IsCollection() ) return false;
-    return $this->collection->is_calendar;
+    if ( !$this->IsCollection() || !isset($this->collection) ) return false;
+    return $this->collection->is_calendar == 't';
+  }
+
+
+  /**
+  * Returns true if the URL referenced by this request points at an addressbook collection.
+  */
+  function IsAddressBook( ) {
+    if ( !$this->IsCollection() || !isset($this->collection) ) return false;
+    return $this->collection->is_addressbook == 't';
   }
 
 
@@ -748,13 +892,20 @@ EOSQL;
   }
 
 
+  /**
+  * Returns the ID of the collection of, or containing this request
+  */
+  function CollectionId( ) {
+    return $this->collection_id;
+  }
+
+
   /**
   * Returns the array of supported privileges converted into XMLElements
   */
-  function RenderSupportedPrivileges( $privs = null ) {
-    global $reply;
+  function BuildSupportedPrivileges( &$reply, $privs = null ) {
     $privileges = array();
-    if ( $privs === null ) $privs = $this->supported_privileges;    
+    if ( $privs === null ) $privs = $this->supported_privileges;
     foreach( $privs AS $k => $v ) {
       dbg_error_log( 'caldav', 'Adding privilege "%s" which is "%s".', $k, $v );
       $privilege = new XMLElement('privilege');
@@ -762,8 +913,8 @@ EOSQL;
       $privset = array($privilege);
       if ( is_array($v) ) {
         dbg_error_log( 'caldav', '"%s" is a container of sub-privileges.', $k );
-        $privset = array_merge($privset, $this->RenderSupportedPrivileges($v));
-      } 
+        $privset = array_merge($privset, $this->BuildSupportedPrivileges($reply,$v));
+      }
       else if ( $v == 'abstract' ) {
         dbg_error_log( 'caldav', '"%s" is an abstract privilege.', $v );
         $privset[] = new XMLElement('abstract');
@@ -777,81 +928,6 @@ EOSQL;
   }
 
 
-  /**
-  * Returns the array of privilege names converted into XMLElements
-  */
-  function RenderPrivileges($privilege_names) {
-    global $reply;
-    $privileges = array();
-    foreach( $privilege_names AS $k => $v ) {
-      dbg_error_log( 'caldav', 'Adding privilege "%s" which is "%s".', $k, $v );
-      $privilege = new XMLElement('privilege');
-      $reply->NSElement($privilege,$k);
-      $privileges[] = $privilege;
-    }
-    return $privileges;
-  }
-
-
-  /**
-  * Return general server-related properties for this URL
-  */
-  function ServerProperty( $tag, $prop, $reply = null ) {
-    global $c, $session;
-
-    if ( $reply === null ) $reply = $GLOBALS['reply'];
-
-    switch( $tag ) {
-      case 'DAV::current-user-principal':
-        $reply->DAVElement( $prop, 'current-user-principal', $this->current_user_principal_xml);
-        break;
-
-      case 'DAV::getcontentlanguage':
-        $locale = (isset($c->current_locale) ? $c->current_locale : '');
-        if ( isset($session->locale) && $session->locale != '' ) $locale = $session->locale;
-        $prop->NewElement('getcontentlanguage', $locale );
-        break;
-
-      case 'DAV::supportedlock':
-        $prop->NewElement('supportedlock',
-          new XMLElement( 'lockentry',
-            array(
-              new XMLElement('lockscope', new XMLElement('exclusive')),
-              new XMLElement('locktype',  new XMLElement('write')),
-            )
-          )
-        );
-        break;
-
-      case 'DAV::acl':
-        /**
-        * @todo This information is semantically valid but presents an incorrect picture.
-        */
-        dbg_error_log( 'caldav', 'Processing "%s" on "%s".', $tag, $this->path );
-        $principal = new XMLElement('principal');
-        $principal->NewElement('authenticated');
-        $grant = new XMLElement( 'grant', array($this->RenderPrivileges($this->permissions)) );
-        $prop->NewElement('acl', new XMLElement( 'ace', array( $principal, $grant ) ) );
-        break;
-
-      case 'DAV::current-user-privilege-set':
-        dbg_error_log( 'caldav', 'Processing "%s" on "%s".', $tag, $this->path );
-        $prop->NewElement('current-user-privilege-set', $this->RenderPrivileges($this->permissions) );
-        break;
-
-      case 'DAV::supported-privilege-set':
-        dbg_error_log( 'caldav', 'Processing "%s" on "%s".', $tag, $this->path );
-        $prop->NewElement('supported-privilege-set', $this->RenderSupportedPrivileges() );
-        break;
-
-      default:
-        dbg_error_log( 'caldav', 'Request for unsupported property "%s" of path "%s".', $tag, $this->path );
-        return false;
-    }
-    return true;
-  }
-
-
   /**
   * Are we allowed to do the requested activity
   *
@@ -867,12 +943,13 @@ EOSQL;
   */
   function AllowedTo( $activity ) {
     global $session;
-    dbg_error_log('session', 'Checking whether "%s" is allowed to "%s"', $session->username, $activity);
-    foreach( $this->permissions AS $k => $v ) {
-      dbg_error_log('session', 'Permissions "%s" is "%s"', $k, $v);
-    }
+    dbg_error_log('caldav', 'Checking whether "%s" is allowed to "%s"', $session->username, $activity);
     if ( isset($this->permissions['all']) ) return true;
     switch( $activity ) {
+      case 'all':
+        return false; // If they got this far then they don't
+        break;
+
       case "CALDAV:schedule-send-freebusy":
         return isset($this->permissions['read']) || isset($this->permissions['urn:ietf:params:xml:ns:caldav:read-free-busy']);
         break;
@@ -912,11 +989,12 @@ EOSQL;
         if ( $this->path == '/' ) return false;
         break;
 
-      case 'read':
-      case 'lock':
-      case 'unlock':
       default:
-        return isset($this->permissions[$activity]);
+        $test_bits = privilege_to_bits( $activity );
+//        dbg_error_log( 'caldav', 'request::AllowedTo("%s") (%s) against allowed "%s" => "%s" (%s)',
+//             (is_array($activity) ? implode(',',$activity) : $activity), decbin($test_bits),
+//             decbin($this->privileges), ($this->privileges & $test_bits), decbin($this->privileges & $test_bits) );
+        return (($this->privileges & $test_bits) > 0 );
         break;
     }
 
@@ -925,6 +1003,26 @@ EOSQL;
 
 
 
+  /**
+  * Return the privileges bits for the current session user to this resource
+  */
+  function Privileges() {
+    return $this->privileges;
+  }
+
+
+  /**
+  * Is the user has the privileges to do what is requested.
+  */
+  function HavePrivilegeTo( $do_what ) {
+    $test_bits = privilege_to_bits( $do_what );
+//    dbg_error_log( 'caldav', 'request::HavePrivilegeTo("%s") [%s] against allowed "%s" => "%s" (%s)',
+//             (is_array($do_what) ? implode(',',$do_what) : $do_what), decbin($test_bits),
+//              decbin($this->privileges), ($this->privileges & $test_bits), decbin($this->privileges & $test_bits) );
+    return ($this->privileges & $test_bits) > 0;
+  }
+
+
   /**
   * Sometimes it's a perfectly formed request, but we just don't do that :-(
   * @param array $unsupported An array of the properties we don't support.
@@ -943,6 +1041,62 @@ EOSQL;
     }
   }
 
+
+  /**
+  * Send a need-privileges error response.  This function will only return
+  * if the $href is not supplied and the current user has the specified
+  * permission for the request path.
+  *
+  * @param string $privilege The name of the needed privilege.
+  * @param string $href The unconstructed URI where we needed the privilege.
+  */
+  function NeedPrivilege( $privileges, $href=null ) {
+    if ( is_string($privileges) ) $privileges = array( $privileges );
+    if ( !isset($href) ) {
+      if ( $this->HavePrivilegeTo($privileges) ) return;
+      $href = $this->path;
+    }
+
+    $reply = new XMLDocument( array('DAV:' => '') );
+    $privnodes = array( $reply->href(ConstructURL($href)), new XMLElement( 'privilege' ) );
+    // RFC3744 specifies that we can only respond with one needed privilege, so we pick the first.
+    $reply->NSElement( $privnodes[1], $privileges[0] );
+    $xml = new XMLElement( 'need-privileges', new XMLElement( 'resource', $privnodes) );
+    $xmldoc = $reply->Render('error',$xml);
+    $this->DoResponse( 403, $xmldoc, 'text/xml; charset="utf-8"' );
+    exit(0);  // Unecessary, but might clarify things
+  }
+
+
+  /**
+  * Send an error response for a failed precondition.
+  *
+  * @param int $status The status code for the failed precondition.  Normally 403
+  * @param string $precondition The namespaced precondition tag.
+  * @param string $explanation An optional text explanation for the failure.
+  */
+  function PreconditionFailed( $status, $precondition, $explanation = '') {
+    $xmldoc = sprintf('<?xml version="1.0" encoding="utf-8" ?>
+<error xmlns="DAV:">
+  <%s/>%s
+</error>', $precondition, $explanation );
+
+    $this->DoResponse( $status, $xmldoc, 'text/xml; charset="utf-8"' );
+    exit(0);  // Unecessary, but might clarify things
+  }
+
+
+  /**
+  * Send a simple error informing the client that was a malformed request
+  *
+  * @param string $text An optional text description of the failure.
+  */
+  function MalformedRequest( $text = 'Bad request' ) {
+    $this->DoResponse( 400, $text );
+    exit(0);  // Unecessary, but might clarify things
+  }
+
+
   /**
   * Send an XML Response.  This function will never return.
   *
@@ -984,30 +1138,24 @@ EOSQL;
       }
     }
 
+    header( "Content-Length: ".strlen($message) );
     echo $message;
 
-    if ( strlen($message) > 100 || strstr($message, "\n") ) {
-      $message = substr( preg_replace("#\s+#m", ' ', $message ), 0, 100) . (strlen($message) > 100 ? "..." : "");
-    }
+    if ( isset($c->dbg['caldav']) && $c->dbg['caldav'] ) {
+      if ( strlen($message) > 100 || strstr($message, "\n") ) {
+        $message = substr( preg_replace("#\s+#m", ' ', $message ), 0, 100) . (strlen($message) > 100 ? "..." : "");
+      }
 
-    dbg_error_log("caldav", "Status: %d, Message: %s, User: %d, Path: %s", $status, $message, $session->user_no, $this->path);
+      dbg_error_log("caldav", "Status: %d, Message: %s, User: %d, Path: %s", $status, $message, $session->user_no, $this->path);
+    }
+    if ( isset($c->dbg['statistics']) && $c->dbg['statistics'] ) {
+      $script_time = microtime(true) - $c->script_start_time;
+      @dbg_error_log("statistics", "Method: %s, Status: %d, Script: %5.3lfs, Queries: %5.3lfs, URL: %s",
+                         $this->method, $status, $script_time, $c->total_query_time, $this->path);
+    }
 
     exit(0);
   }
 
-
-  /**
-  * Return an array of what the DAV privileges are that are supported
-  *
-  * @return array The supported privileges.
-  */
-  function SupportedPrivileges() {
-    $privs = array( 'all'=>'abstract', 'read'=>'real',
-                    'write'=>'real', 'bind'=>'real',
-                    'unbind'=>'real', 'write-content'=>'real',
-                    'write-properties'=>'real',
-                    'urn:ietf:params:xml:ns:caldav:read-free-busy' => 'real');
-    return $privs;
-  }
 }
 
