@@ -23,25 +23,25 @@ class HTTPAuthSession {
   * User ID number
   * @var user_no int
   */
-  var $user_no;
+  public $user_no;
 
   /**
   * User e-mail
   * @var email string
   */
-  var $email;
+  public $email;
 
   /**
   * User full name
   * @var fullname string
   */
-  var $fullname;
+  public $fullname;
 
   /**
   * Group rights
   * @var groups array
   */
-  var $groups;
+  public $groups;
   /**#@-*/
 
   /**
@@ -228,13 +228,13 @@ class HTTPAuthSession {
       if (isset($c->authenticate_hook['optional']) && $c->authenticate_hook['optional']) {
         if ($hook_response !== false) { return $hook_response; }
       } else {
-	return $hook_response;
+        return $hook_response;
       }
     }
 
     if ( $usr = getUserByName($username) ) {
-      dbg_error_log( "BasicAuth", ":CheckPassword: Name:%s, Pass:%s, File:%s", $username, $password, $usr->password );
-      if ( session_validate_password( $password, $usr->password ) ) {
+      dbg_error_log( "BasicAuth", ":CheckPassword: Name:%s, Pass:%s, File:%s, Active:%s", $username, $password, $usr->password, ($usr->active?'Yes':'No') );
+      if ( $usr->active && session_validate_password( $password, $usr->password ) ) {
         return $usr;
       }
     }
@@ -259,8 +259,9 @@ class HTTPAuthSession {
   */
   function GetRoles () {
     $this->roles = array();
-    $qry = new PgQuery( 'SELECT role_name FROM role_member m join roles r ON r.role_no = m.role_no WHERE user_no = ? ', $this->user_no );
-    if ( $qry->Exec('BasicAuth') && $qry->rows > 0 ) {
+    $qry = new AwlQuery( 'SELECT role_name FROM role_member m join roles r ON r.role_no = m.role_no WHERE user_no = :user_no ',
+                                array( ':user_no' => $this->user_no) );
+    if ( $qry->Exec('BasicAuth') && $qry->rows() > 0 ) {
       while( $role = $qry->Fetch() ) {
         $this->roles[$role->role_name] = true;
       }
