@@ -20,16 +20,14 @@
 *
 * @package   davical
 * @subpackage   authentication
-* @author    Andrew McMillan <andrew@catalyst.net.nz>
-* @copyright Catalyst IT Ltd
-* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
+* @author    Andrew McMillan <andrew@mcmillan.net.nz>
+* @copyright Catalyst IT Ltd, Morphoss Ltd
+* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2 or later
 */
 
-if ( function_exists('CreateHomeCalendar') ) return(true);
-
-require_once("AWLUtilities.php");
 require_once("DataUpdate.php");
 
+
 /**
 * Create a default home calendar for the user.
 * @param string $username The username of the user we are creating relationships for.
@@ -42,9 +40,17 @@ function CreateHomeCalendar( $username ) {
   $parent_path = "/".$username."/";
   $calendar_path = $parent_path . $c->home_calendar_name."/";
   $dav_etag = md5($usr->user_no . $calendar_path);
-  $sql = "INSERT INTO collection (user_no, parent_container, dav_name, dav_etag, dav_displayname, is_calendar, ";
-  $sql .= "created, modified) VALUES( ?, ?, ?, ?, ?, true, current_timestamp, current_timestamp );";
-  $qry = new PgQuery( $sql, $usr->user_no, $parent_path, $calendar_path, $dav_etag, $usr->fullname);
+  $sql = 'INSERT INTO collection (user_no, parent_container, dav_name, dav_etag, dav_displayname, is_calendar, created, modified, resourcetypes) ';
+  $sql .= 'VALUES( :user_no, :parent_container, :calendar_path, :dav_name, :dav_etag, true, current_timestamp, current_timestamp, :resourcetypes );';
+  $params = array(
+      ':user_no' => $usr->user_no,
+      ':parent_container' => $parent_path,
+      ':calendar_path' => $calendar_path,
+      ':dav_name' => $dav_etag,
+      ':dav_etag' => $usr->fullname,
+      ':resourcetypes' => '<DAV::collection/><urn:ietf:params:xml:ns:caldav:calendar/>'
+  );
+  $qry = new AwlQuery( $sql, $params );
   if ( $qry->Exec() ) {
     $c->messages[] = i18n("Home calendar added.");
     dbg_error_log("User",":Write: Created user's home calendar at '%s'", $calendar_path );
@@ -58,28 +64,10 @@ function CreateHomeCalendar( $username ) {
 
 
 /**
-* Create default relationships
+* Defunct function for creating default relationships.
 * @param string $username The username of the user we are creating relationships for.
 */
 function CreateDefaultRelationships( $username ) {
-  global $session, $c;
-  if ( ! isset($c->default_relationships) || !is_array($c->default_relationships) || count($c->default_relationships) == 0 ) return false;
-
-  $usr = getUserByName( $username );
-  $sql = "";
-  foreach( $c->default_relationships AS $to_user => $permission ) {
-    $sql .= "INSERT INTO relationship (from_user, to_user, rt_id) ";
-    $sql .= "VALUES( $usr->user_no, $to_user, (select rt_id from relationship_type where confers = '$permission' order by rt_id limit 1) );";
-  }
-  $qry = new PgQuery( $sql );
-  if ( $qry->Exec() ) {
-    $c->messages[] = i18n("Default relationships added.");
-    dbg_error_log("User",":Write: Added default relationships" );
-  }
-  else {
-    $c->messages[] = i18n("There was an error writing to the database.");
-    return false;
-  }
   return true;
 }
 
@@ -89,18 +77,19 @@ function CreateDefaultRelationships( $username ) {
 * @param object $usr The user details we read from the remote.
 */
 function UpdateUserFromExternal( &$usr ) {
+  global $c;
   /**
   * When we're doing the create we will usually need to generate a user number
   */
   if ( !isset($usr->user_no) || intval($usr->user_no) == 0 ) {
-    $qry = new PgQuery( "SELECT nextval('usr_user_no_seq');" );
-    $qry->Exec('Login',__LINE,__FILE__);
+    $qry = new AwlQuery( "SELECT nextval('usr_user_no_seq');" );
+    $qry->Exec('Login',__LINE__,__FILE__);
     $sequence_value = $qry->Fetch(true);  // Fetch as an array
     $usr->user_no = $sequence_value[0];
   }
 
-  $qry = new PgQuery("SELECT * FROM usr WHERE user_no = $usr->user_no;" );
-  if ( $qry->Exec('Login',__LINE,__FILE__) && $qry->rows == 1 ) {
+  $qry = new AwlQuery('SELECT * FROM usr WHERE user_no = :user_no', array(':user_no' => $usr->user_no) );
+  if ( $qry->Exec('Login',__LINE__,__FILE__) && $qry->rows() == 1 ) {
     $type = "UPDATE";
     if ( $old = $qry->Fetch() ) {
       $changes = false;
@@ -124,17 +113,21 @@ function UpdateUserFromExternal( &$usr ) {
   else
     $type = "INSERT";
 
-  $qry = new PgQuery( sql_from_object( $usr, $type, 'usr', "WHERE user_no=$usr->user_no" ) );
-  $qry->Exec('Login',__LINE,__FILE__);
+  $params = array();
+  if ( $type != 'INSERT' ) $params[':user_no'] = $usr->user_no;
+  $qry = new AwlQuery( sql_from_object( $usr, $type, 'usr', 'WHERE user_no= :user_no' ), $params );
+  $qry->Exec('Login',__LINE__,__FILE__);
 
   /**
   * We disallow login by inactive users _after_ we have updated the local copy
   */
-  if ( isset($usr->active) && $usr->active == 'f' ) return false;
+  if ( isset($usr->active) && ($usr->active === 'f' || $usr->active === false) ) return false;
 
   if ( $type == 'INSERT' ) {
+    $qry = new AwlQuery( 'INSERT INTO principal( type_id, user_no, displayname, default_privileges) SELECT 1, user_no, fullname, :privs::INT::BIT(24) FROM usr WHERE username=:username',
+                          array( ':privs' => privilege_to_bits($c->default_privileges), ':username' => $usr->username) );
+    $qry->Exec('Login',__LINE__,__FILE__);
     CreateHomeCalendar($usr->username);
-    CreateDefaultRelationships($usr->username);
   }
 }
 
@@ -145,7 +138,7 @@ function UpdateUserFromExternal( &$usr ) {
 *
 * Use this as in the following example config snippet:
 *
-*  include('auth-functions.php');
+* require_once('auth-functions.php');
 *  $c->authenticate_hook = array(
 *      'call'   => 'AuthExternalAwl',
 *      'config' => array(
@@ -163,7 +156,19 @@ function AuthExternalAWL( $username, $password ) {
   global $c;
 
   $persistent = isset($c->authenticate_hook['config']['use_persistent']) && $c->authenticate_hook['config']['use_persistent'];
-  $authconn = ( $persistent ? pg_pConnect($c->authenticate_hook['config']['connection']) : pg_Connect($c->authenticate_hook['config']['connection']));
+
+  if ( isset($c->authenticate_hook['config']['columns']) )
+    $cols = $c->authenticate_hook['config']['columns'];
+  else
+    $cols = '*';
+
+  if ( isset($c->authenticate_hook['config']['where']) )
+    $andwhere = ' AND '.$c->authenticate_hook['config']['where'];
+  else
+    $andwhere = '';
+
+  $qry = new AwlQuery('SELECT '.$cols.' FROM usr WHERE lower(username) = :username '. $andwhere, array( ':username' => strtolower($username) ));
+  $authconn = $qry->SetConnection($c->authenticate_hook['config']['connection'], ($persistent ? array(PDO::ATTR_PERSISTENT => true) : null));
   if ( ! $authconn ) {
     echo <<<EOERRMSG
   <html><head><title>Database Connection Failure</title></head><body>
@@ -175,19 +180,7 @@ EOERRMSG;
     exit(1);
   }
 
-  if ( isset($c->authenticate_hook['config']['columns']) )
-    $cols = $c->authenticate_hook['config']['columns'];
-  else
-    $cols = "*";
-
-  if ( isset($c->authenticate_hook['config']['where']) )
-    $andwhere = " AND ".$c->authenticate_hook['config']['where'];
-  else
-    $andwhere = "";
-
-  $qry = new PgQuery("SELECT $cols FROM usr WHERE lower(username) = ? $andwhere", strtolower($username) );
-  $qry->SetConnection($authconn);
-  if ( $qry->Exec('Login',__LINE,__FILE__) && $qry->rows == 1 ) {
+  if ( $qry->Exec('Login',__LINE__,__FILE__) && $qry->rows() == 1 ) {
     $usr = $qry->Fetch();
     if ( session_validate_password( $password, $usr->password ) ) {
       UpdateUserFromExternal($usr);
