@@ -1,5 +1,21 @@
 <?php
 
+$need_expansion = false;
+function check_for_expansion( $calendar_data_node ) {
+  global $need_expansion, $expand_range_start, $expand_range_end;
+
+  if ( !class_exists('DateTime') ) return; /** We don't support expansion on PHP5.1 */
+
+  $expansion = $calendar_data_node->GetElements('urn:ietf:params:xml:ns:caldav:expand');
+  if ( isset($expansion[0]) ) {
+    $need_expansion = true;
+    $expand_range_start = $expansion[0]->GetAttribute('start');
+    $expand_range_end = $expansion[0]->GetAttribute('end');
+    if ( isset($expand_range_start) ) $expand_range_start = new RepeatRuleDateTime($expand_range_start);
+    if ( isset($expand_range_end) )   $expand_range_end   = new RepeatRuleDateTime($expand_range_end);
+  }
+}
+
 /**
  * Build the array of properties to include in the report output
  */
@@ -8,15 +24,22 @@ $proptype = $qry_content[0]->GetTag();
 $properties = array();
 switch( $proptype ) {
   case 'DAV::prop':
-    $qry_props = $xmltree->GetPath('/urn:ietf:params:xml:ns:caldav:calendar-query/DAV::prop/*');
-    foreach( $qry_props AS $k => $v ) {
+    $qry_props = $xmltree->GetPath('/urn:ietf:params:xml:ns:caldav:calendar-query/'.$proptype.'/*');
+    foreach( $qry_content[0]->GetElements() AS $k => $v ) {
       $propertyname = preg_replace( '/^.*:/', '', $v->GetTag() );
       $properties[$propertyname] = 1;
+      if ( $v->GetTag() == 'urn:ietf:params:xml:ns:caldav:calendar-data' ) check_for_expansion($v);
     }
     break;
 
   case 'DAV::allprop':
     $properties['allprop'] = 1;
+    if ( $qry_content[1]->GetTag() == 'DAV::include' ) {
+      foreach( $qry_content[1]->GetElements() AS $k => $v ) {
+        $include_properties[] = $v->GetTag(); /** $include_properties is referenced in DAVResource where allprop is expanded */
+        if ( $v->GetTag() == 'urn:ietf:params:xml:ns:caldav:calendar-data' ) check_for_expansion($v);
+      }
+    }
     break;
 
   default:
@@ -70,8 +93,9 @@ function apply_filter( $filters, $item ) {
  */
 $need_post_filter = false;
 function SqlFilterFragment( $filter, $components, $property = null, $parameter = null ) {
-  global $need_post_filter;
+  global $need_post_filter, $target_collection;
   $sql = "";
+  $params = array();
   if ( !is_array($filter) ) {
     dbg_error_log( "calquery", "Filter is of type '%s', but should be an array of XML Tags.", gettype($filter) );
   }
@@ -97,7 +121,9 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
             case 'dtend':
             case 'dtstamp':
             case 'dtstart':
-              $property_defined_match = "IS NOT NULL";
+              if ( ! $target_collection->IsSchedulingCollection() ) {
+                $property_defined_match = "IS NOT NULL";
+              }
               break;
 
             case 'priority':
@@ -120,8 +146,9 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
         $start = $v->GetAttribute("start");
         $finish = $v->GetAttribute("end");
         if ( isset($start) || isset($finish) ) {
-          $sql .= "AND (rrule_event_overlaps( dtstart, dtend, rrule, ".qpg($start).", ".qpg($finish)." ) ";
-          $sql .= sprintf( "OR event_has_exceptions(caldav_data.caldav_data) )" );
+          $sql .= ' AND (rrule_event_overlaps( dtstart, dtend, rrule, :time_range_start, :time_range_end ) OR event_has_exceptions(caldav_data.caldav_data) ) ';
+          $params[':time_range_start'] = $start;
+          $params[':time_range_end'] = $finish;
         }
         break;
 
@@ -138,22 +165,27 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
             $comparison = 'ILIKE';
             break;
         }
-        dbg_error_log("calquery", " text-match: (%s IS NULL OR %s%s %s %s) ", $property, (isset($negate) && strtolower($negate) == "yes" ? "NOT ": ""),
-                                          $property, $comparison, qpg("%".$search."%") );
-        $sql .= sprintf( "AND (%s IS NULL OR %s%s %s %s) ", $property, (isset($negate) && strtolower($negate) == "yes" ? "NOT ": ""),
-                                          $property, $comparison, qpg("%".$search."%") );
+        $params[':text_match'] = '%'.$search.'%';
+        dbg_error_log("calquery", " text-match: (%s IS NULL OR %s%s %s '%s') ", $property, (isset($negate) && strtolower($negate) == "yes" ? "NOT ": ""),
+                                          $property, $comparison, $params[':text_match'] );
+        $sql .= sprintf( "AND (%s IS NULL OR %s%s %s :text_match) ", $property, (isset($negate) && strtolower($negate) == "yes" ? "NOT ": ""),
+                                          $property, $comparison );
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:comp-filter':
         $comp_filter_name = $v->GetAttribute("name");
         if ( count($components) == 0 ) {
-          $sql .= "AND caldav_data.caldav_type = ".qpg($comp_filter_name)." ";
+          $sql .= "AND caldav_data.caldav_type = :component_name_filter ";
+          $params[':component_name_filter'] = $comp_filter_name;
         }
         $components[] = $comp_filter_name;
         $subfilter = $v->GetContent();
         if ( is_array( $subfilter ) ) {
           $success = SqlFilterFragment( $subfilter, $components, $property, $parameter );
-          if ( $success === false ) continue; else $sql .= $success;
+          if ( $success === false ) continue; else {
+            $sql .= $success['sql'];
+            $params = array_merge( $params, $success['params'] );
+          }
         }
         break;
 
@@ -190,7 +222,10 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
         }
         $subfilter = $v->GetContent();
         $success = SqlFilterFragment( $subfilter, $components, $property, $parameter );
-        if ( $success === false ) continue; else $sql .= $success;
+        if ( $success === false ) continue; else {
+          $sql .= $success['sql'];
+          $params = array_merge( $params, $success['params'] );
+        }
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:param-filter':
@@ -199,12 +234,19 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
         $parameter = $v->GetAttribute("name");
         $subfilter = $v->GetContent();
         $success = SqlFilterFragment( $subfilter, $components, $property, $parameter );
-        if ( $success === false ) continue; else $sql .= $success;
+        if ( $success === false ) continue; else {
+          $sql .= $success['sql'];
+          $params = array_merge( $params, $success['params'] );
+        }
+        break;
+
+      default:
+        dbg_error_log("calquery", "Could not handle unknown tag '%s' in calendar query report", $tag );
         break;
     }
   }
   dbg_error_log("calquery", "Generated SQL was '%s'", $sql );
-  return $sql;
+  return array( 'sql' => $sql, 'params' => $params );
 }
 
 /**
@@ -217,9 +259,7 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
  */
 function BuildSqlFilter( $filter ) {
   $components = array();
-  $sql = SqlFilterFragment( $filter, $components );
-  if ( $sql === false ) return "";
-  return $sql;
+  return SqlFilterFragment( $filter, $components );
 }
 
 
@@ -228,6 +268,14 @@ function BuildSqlFilter( $filter ) {
 */
 
 $responses = array();
+$target_collection = new DAVResource($request->path);
+$bound_from = $target_collection->bound_from();
+if ( !$target_collection->Exists() ) {
+  $request->DoResponse( 404 );
+}
+if ( ! ($target_collection->IsCalendar() || $target_collection->IsSchedulingCollection()) ) {
+  $request->DoResponse( 403, translate('The calendar-query report must be run against a calendar or a scheduling collection') );
+}
 
 /**
 * @todo Once we are past DB version 1.2.1 we can change this query more radically.  The best performance to
@@ -237,25 +285,43 @@ $responses = array();
 *              AND caldav_data.caldav_type = 'VEVENT' ORDER BY caldav_data.user_no, caldav_data.dav_name;
 */
 
-$where = " WHERE caldav_data.user_no = $request->user_no AND caldav_data.dav_name ~ ".qpg("^".$request->path)." ";
+$params = array();
+$where = ' WHERE caldav_data.collection_id = ' . $target_collection->resource_id();
 if ( is_array($qry_filters) ) {
   dbg_log_array( "calquery", "qry_filters", $qry_filters, true );
-  $where .= BuildSqlFilter( $qry_filters );
+  $components = array();
+  $filter_fragment =  SqlFilterFragment( $qry_filters, $components );
+  if ( $filter_fragment !== false ) {
+    $where .= ' '.$filter_fragment['sql'];
+    $params = $filter_fragment['params'];
+  }
 }
-if ( ! $request->AllowedTo('all') ) {
-  $where .= "AND (calendar_item.class != 'PRIVATE' OR calendar_item.class IS NULL) ";
+if ( $target_collection->Privileges() != privilege_to_bits('DAV::all') ) {
+  $where .= " AND (calendar_item.class != 'PRIVATE' OR calendar_item.class IS NULL) ";
 }
 
-if ( isset($c->hide_TODO) && $c->hide_TODO && ! $request->AllowedTo('all') ) {
-  $where .= "AND caldav_data.caldav_type NOT IN ('VTODO') ";
+if ( isset($c->hide_TODO) && $c->hide_TODO && ! $target_collection->HavePrivilegeTo('DAV::write-content') ) {
+  $where .= " AND caldav_data.caldav_type NOT IN ('VTODO') ";
 }
 
-$sql = "SELECT * FROM caldav_data INNER JOIN calendar_item USING(dav_id,user_no,dav_name)". $where;
+if ( isset($c->hide_older_than) && intval($c->hide_older_than > 0) ) {
+  $where .= " AND calendar_item.dtstart > (now() - interval '".intval($c->hide_older_than)." days') ";
+}
+
+$sql = 'SELECT * FROM caldav_data INNER JOIN calendar_item USING(dav_id,user_no,dav_name)'. $where;
 if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY dav_id";
-$qry = new PgQuery( $sql );
-if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows > 0 ) {
+$qry = new AwlQuery( $sql, $params );
+if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows() > 0 ) {
   while( $calendar_object = $qry->Fetch() ) {
     if ( !$need_post_filter || apply_filter( $qry_filters, $calendar_object ) ) {
+      if ( $bound_from != $target_collection->dav_name() ) {
+        $calendar_object->dav_name = str_replace( $bound_from, $target_collection->dav_name(), $calendar_object->dav_name);
+      }
+      if ( $need_expansion ) {
+        $ics = new iCalComponent($calendar_object->caldav_data);
+        $expanded = expand_event_instances($ics, $expand_range_start, $expand_range_end);
+        $calendar_object->caldav_data = $expanded->Render();
+      }
       $responses[] = calendar_to_xml( $properties, $calendar_object );
     }
   }
