@@ -7,10 +7,10 @@
 * @subpackage   ldap
 * @author    Maxime Delorme <mdelorme@tennaxia.net>
 * @copyright Maxime Delorme
-* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
+* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2 or later
 */
 
-if ( !function_exists('CreateHomeCalendar') ) include('auth-functions.php');
+require_once("auth-functions.php");
 
 class ldapDrivers
 {
@@ -63,7 +63,7 @@ class ldapDrivers
           $this->connect=ldap_connect($host);
 
       if (! $this->connect){
-          $c->messages[] = sprintf(i18n( "drivers_ldap : Unable to connect to LDAP with port %s on host %s"), $port,$host );
+          $c->messages[] = sprintf(translate( 'drivers_ldap : Unable to connect to LDAP with port %s on host %s'), $port, $host );
           $this->valid=false;
           return ;
       }
@@ -73,12 +73,12 @@ class ldapDrivers
       // Start TLS if desired (requires protocol version 3)
       if (isset($config['startTLS'])) {
         if (!ldap_set_option($this->connect, LDAP_OPT_PROTOCOL_VERSION, 3)) {
-          $c->messages[] = sprintf(i18n( "drivers_ldap : Failed to set LDAP to use protocol version 3, TLS not supported") );
+          $c->messages[] = i18n('drivers_ldap : Failed to set LDAP to use protocol version 3, TLS not supported');
           $this->valid=false;
           return;
         }
         if (!ldap_start_tls($this->connect)) {
-          $c->messages[] = sprintf(i18n( "drivers_ldap : Could not start TLS: ldap_start_tls() failed") );
+          $c->messages[] = i18n('drivers_ldap : Could not start TLS: ldap_start_tls() failed');
           $this->valid=false;
           return;
         }
@@ -114,7 +114,7 @@ class ldapDrivers
       //root to start search
       $this->baseDNUsers  = is_string($config['baseDNUsers']) ? array($config['baseDNUsers']) : $config['baseDNUsers'];
       $this->filterUsers  = (isset($config['filterUsers'])  ? $config['filterUsers']  : null);
-      $this->baseDNGroups = (isset($config['baseDNGroups']) ? $config['baseDNGroups'] : null);
+      $this->baseDNGroups = is_string($config['baseDNGroups']) ? array($config['baseDNGroups']) : $config['baseDNGroups'];
       $this->filterGroups = (isset($config['filterGroups']) ? $config['filterGroups'] : null);
   }
 
@@ -129,9 +129,13 @@ class ldapDrivers
     foreach($this->baseDNUsers as $baseDNUsers) {
       $entry = $query($this->connect,$baseDNUsers,$this->filterUsers,$attributes);
 
-      if (!ldap_first_entry($this->connect,$entry))
-        $c->messages[] = sprintf(i18n("Error NoUserFound with filter >%s<, attributes >%s< , dn >%s<"),$this->filterUsers,join(', ',$attributes), $baseDNUsers);
-
+      if (!ldap_first_entry($this->connect,$entry)) {
+        $c->messages[] = sprintf(translate('Error NoUserFound with filter >%s<, attributes >%s< , dn >%s<'),
+                                 $this->filterUsers,
+                                 join(', ', $attributes),
+                                 $baseDNUsers);
+      }
+      $row = array();
       for($i = ldap_first_entry($this->connect,$entry);
           $i && $arr = ldap_get_attributes($this->connect,$i);
           $i = ldap_next_entry($this->connect,$i) ) {
@@ -143,6 +147,36 @@ class ldapDrivers
     }
     return $ret;
   }
+  
+  /**
+  * Retrieve all groups from the LDAP directory
+  */
+  function getAllGroups($attributes){
+    global $c;
+
+    $query = $this->ldap_query_all;
+
+    foreach($this->baseDNGroups as $baseDNGroups) {
+      $entry = $query($this->connect,$baseDNGroups,$this->filterGroups,$attributes);
+
+      if (!ldap_first_entry($this->connect,$entry)) {
+        $c->messages[] = sprintf(translate('Error NoGroupFound with filter >%s<, attributes >%s< , dn >%s<'),
+                                 $this->filterUsers,
+                                 join(', ', $attributes),
+                                 $baseDNGroups);
+      }
+      $row = array();
+      for($i = ldap_first_entry($this->connect,$entry);
+          $i && $arr = ldap_get_attributes($this->connect,$i);
+          $i = ldap_next_entry($this->connect,$i) ) {
+        for ($j=0; $j < $arr['count']; $j++) {
+          $row[$arr[$j]] = count($arr[$arr[$j]])>2?$arr[$arr[$j]]:$arr[$arr[$j]][0];
+        }
+        $ret[]=$row;
+      }
+    }
+    return $ret;
+  }
 
   /**
     * Returns the result of the LDAP query
@@ -318,6 +352,112 @@ function LDAP_check($username, $password ){
 
 }
 
+/**
+* sync LDAP Groups against the DB
+*/
+function sync_LDAP_groups(){
+  global $c;
+  $ldapDriver = getStaticLdap();
+  if($ldapDriver->valid){
+    $mapping = $c->authenticate_hook['config']['group_mapping_field'];
+    //$attributes = array('cn','modifyTimestamp','memberUid');
+    $attributes = array_values($mapping);
+    $ldap_groups_tmp = $ldapDriver->getAllGroups($attributes);
+    
+    if ( sizeof($ldap_groups_tmp) == 0 )
+      return;
+
+    foreach($ldap_groups_tmp as $key => $ldap_group){
+      $ldap_groups_info[$ldap_group[$mapping['username']]] = $ldap_group;
+      unset ( $ldap_groups_info[$ldap_group[$mapping['username']]][$mapping['members']]['count'] );
+      unset($ldap_groups_tmp[$key]);
+    }
+    $db_groups = array ();
+    $db_group_members = array ();
+    $qry = new AwlQuery( "SELECT g.username AS group_name, member.username AS member_name FROM dav_principal g JOIN group_member ON (g.principal_id=group_member.group_id) JOIN dav_principal member  ON (member.principal_id=group_member.member_id) WHERE g.type_id = 3");
+    $qry->Exec('sync_LDAP',__LINE__,__FILE__);
+    while($db_group = $qry->Fetch()) {
+      $db_groups[$db_group->group_name] = $db_group->group_name;
+      $db_group_members[$db_group->group_name][] = $db_group->member_name;
+    }
+    
+    $ldap_groups = array_keys($ldap_groups_info);
+    // users only in ldap
+    $groups_to_create = array_diff($ldap_groups,$db_groups);
+    // users only in db
+    $groups_to_deactivate = array_diff($db_groups,$ldap_groups);
+    // users present in ldap and in the db
+    $groups_to_update = array_intersect($db_groups,$ldap_groups);
+    
+    if ( sizeof ( $groups_to_create ) ){
+      $c->messages[] = sprintf(i18n('- creating groups : %s'),join(', ',$groups_to_create));
+      $validUserFields = get_fields('usr');
+      foreach ( $groups_to_create as $k => $group ){
+        $user = (object) array( 'user_no' => 0, 'username' => '' );
+    
+        if ( isset($c->authenticate_hook['config']['default_value']) && is_array($c->authenticate_hook['config']['default_value']) ) {
+          foreach ( $c->authenticate_hook['config']['default_value'] as $field => $value ) {
+            if ( isset($validUserFields[$field]) ) {
+              $usr->{$field} =  $value;
+              dbg_error_log( "LDAP", "Setting usr->%s to %s from configured defaults", $field, $value );
+            }
+          }
+        }
+        $user->user_no = 0;
+        foreach ( $mapping as $field => $value ) {
+          dbg_error_log( "LDAP", "Considering copying %s", $field );
+          if ( isset($validUserFields[$field]) ) {
+            $user->{$field} =  $ldap_values[$value];
+            dbg_error_log( "LDAP", "Setting usr->%s to %s from LDAP field %s", $field, $ldap_values[$value], $value );
+          }
+        }
+        $user->fullname = $group;  
+        $user->displayname = $group;  
+        $user->username = $group;  
+        UpdateUserFromExternal( $user );
+        $qry = new AwlQuery( "UPDATE dav_principal set type_id = 3 WHERE username=:group ",array(':group'=>$group) );
+        $qry->Exec('sync_LDAP',__LINE__,__FILE__);
+        $c->messages[] = sprintf(i18n('- adding users %s to group : %s'),join(',',$ldap_groups_info[$group][$mapping['members']]),$group);
+        foreach ( $ldap_groups_info[$group][$mapping['members']] as $member ){
+          $qry = new AwlQuery( "INSERT INTO group_member SELECT g.principal_id AS group_id,u.principal_id AS member_id FROM dav_principal g, dav_principal u WHERE g.username=:group AND u.username=:member;",array (':group'=>$group,':member'=>$member) );
+          $qry->Exec('sync_LDAP_groups',__LINE__,__FILE__);
+        }
+      }
+    }
+
+    if ( sizeof ( $groups_to_update ) ){
+      $c->messages[] = sprintf(i18n('- updating groups : %s'),join(', ',$groups_to_update));
+      foreach ( $groups_to_update as $group ){
+        $db_members = array_values ( $db_group_members[$group] );
+        $ldap_members = array_values ( $ldap_groups_info[$group][$mapping['members']] );
+        $add_users = array_diff ( $ldap_members, $db_members );
+        if ( sizeof ( $add_users ) ){
+          $c->messages[] = sprintf(i18n('- adding %s to group : %s'),join(', ', $add_users ), $group);
+          foreach ( $add_users as $member ){
+            $qry = new AwlQuery( "INSERT INTO group_member SELECT g.principal_id AS group_id,u.principal_id AS member_id FROM dav_principal g, dav_principal u WHERE g.username=:group AND u.username=:member",array (':group'=>$group,':member'=>$member) );
+            $qry->Exec('sync_LDAP_groups',__LINE__,__FILE__);
+          }
+        }
+        $remove_users = array_diff ( $db_members, $ldap_members );
+        if ( sizeof ( $remove_users ) ){
+          $c->messages[] = sprintf(i18n('- removing %s from group : %s'),join(', ', $remove_users ), $group);
+          foreach ( $remove_users as $member ){
+            $qry = new AwlQuery( "DELETE FROM group_member USING dav_principal g,dav_principal m WHERE group_id=g.principal_id AND member_id=m.principal_id AND g.username=:group AND m.username=:member",array (':group'=>$group,':member'=>$member) );
+            $qry->Exec('sync_LDAP_groups',__LINE__,__FILE__);
+          }
+        }
+      }
+    }
+
+    if ( sizeof ( $groups_to_deactivate ) ){
+      $c->messages[] = sprintf(i18n('- deactivate groups : %s'),join(', ',$groups_to_deactivate));
+      foreach ( $groups_to_deactivate as $group ){
+        $qry = new AwlQuery( "UPDATE dav_principal set active='f'::bool WHERE username=:group AND type_id = 3",array(':group'=>$group) );
+        $qry->Exec('sync_LDAP',__LINE__,__FILE__);
+      }
+    }
+  }
+}
 
 /**
 * sync LDAP against the DB
@@ -337,13 +477,12 @@ function sync_LDAP(){
       $ldap_users_info[$ldap_user[$mapping["username"]]] = $ldap_user;
       unset($ldap_users_tmp[$key]);
     }
-    $qry = new PgQuery( "SELECT username, user_no, updated FROM usr ");
+    $qry = new AwlQuery( "SELECT username, user_no, modified as updated FROM dav_principal where type_id=1");
     $qry->Exec('sync_LDAP',__LINE__,__FILE__);
-    while($db_user = $qry->Fetch(true)){
-      $db_users[] = $db_user['username'];
-      $db_users_info[$db_user['username']] = array('user_no' => $db_user['user_no'], 'updated' => $db_user['updated']);
+    while($db_user = $qry->Fetch()) {
+      $db_users[] = $db_user->username;
+      $db_users_info[$db_user->username] = array('user_no' => $db_user->user_no, 'updated' => $db_user->updated);
     }
-    include_once("DAViCalUser.php");
 
     $ldap_users = array_keys($ldap_users_info);
     // users only in ldap
@@ -375,13 +514,15 @@ function sync_LDAP(){
     }
 
     // deactivating all users
-    if ( sizeof($users_to_deactivate) ) {
-      foreach( $users_to_deactivate AS $v ) {
-          $usr_in .= ', ' . qpg($v);
-      }
-      $usr_in = substr($usr_in,1);
+    $params = array();
+    $i = 0;
+    foreach( $users_to_deactivate AS $v ) {
+      if ( isset($c->do_not_sync_from_ldap) && isset($c->do_not_sync_from_ldap[$v]) ) continue;
+      $params[':u'.$i++] = strtolower($v);
+    }
+    if ( count($params) > 0 ) {
       $c->messages[] = sprintf(i18n('- deactivating users : %s'),join(', ',$users_to_deactivate));
-      $qry = new PgQuery( "UPDATE usr SET active = FALSE WHERE lower(username) IN ($usr_in)");
+      $qry = new AwlQuery( 'UPDATE usr SET active = FALSE WHERE lower(username) IN ('.implode(',',array_keys($params)).')', $params);
       $qry->Exec('sync_LDAP',__LINE__,__FILE__);
     }
 
@@ -415,6 +556,16 @@ function sync_LDAP(){
         $c->messages[] = sprintf(i18n('- updating user records : %s'),join(', ',$users_to_update));
       if ( sizeof($users_nothing_done) )
         $c->messages[] = sprintf(i18n('- nothing done on : %s'),join(', ', $users_nothing_done));
-    }
+		}
+
+		$admins = 0;
+    $qry = new AwlQuery( "select count(*) as admins from usr join role_member using ( user_no ) join roles using (role_no) where usr.active = true and role_name='Admin'");
+    $qry->Exec('sync_LDAP',__LINE__,__FILE__);
+    while($db_user = $qry->Fetch()) {
+      $admins = $db_user->admins;
+		}
+		if ( $admins == 0 ) {
+      $c->messages[] = sprintf(i18n('Warning: there are no active admin users, you should fix this before logging out.'));
+		}
   }
 }
