@@ -10,7 +10,7 @@
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
 */
 
-if ( !function_exists('CreateHomeCalendar') ) include('auth-functions.php');
+require_once("auth-functions.php");
 
 class squidPamDrivers
 {
@@ -39,7 +39,7 @@ class squidPamDrivers
   {
       global $c;
       if (! file_exists($config)){
-          $c->messages[] = sprintf(i18n( "drivers_squid_pam : Unable to find %s file"), $config );
+          $c->messages[] = sprintf(i18n( 'drivers_squid_pam : Unable to find %s file'), $config );
           $this->valid=false;
           return ;
       }
@@ -67,7 +67,8 @@ function SQUID_PAM_check($username, $password ){
     }
     else {
       dbg_error_log( "PAM", "user %s doesn't exist in local DB, we need to create it",$username );
-      $fullname = trim( exec("getent passwd | grep ^" . $username ." | cut -d \":\" -f5"), ' ,' );
+      $fullname = exec('getent passwd "'.$username.'"' );
+      $fullname = preg_replace( '{^[^:]+:[^:]+:\d+:\d+:([^:,]+)(,[^:]*):.*$}', '$1', $fullname );
       $usr = (object) array(
               'user_no' => 0,
               'username' => $username,
