@@ -1,12 +1,12 @@
 <?php
-require_once("iCalendar.php");
-include_once("RRule.php");
+require_once('iCalendar.php');
+include_once('RRule.php');
 
-if ( isset($request->by_email) ) {
-  $where = "WHERE caldav_data.user_no = $request->user_no ";
-}
-else {
-  $where = "WHERE caldav_data.user_no = $request->user_no AND caldav_data.dav_name ~ ".qpg("^".$request->path)." ";
+$params = array( ':user_no' => $request->user_no );
+$where = 'WHERE caldav_data.user_no = :user_no ';
+if ( ! isset($request->by_email) ) {
+  $params[':path_match'] = "^".$request->path;
+  $where .= 'AND caldav_data.dav_name ~ :path_match ';
 }
 
 $begindate = new iCalDate($fb_start);
@@ -18,8 +18,10 @@ else {
   $enddate->AddDuration( $fb_period );
 }
 
+$params[':range_start'] = $fb_start;
 if ( isset($fb_end) && $fb_end != '' ) {
-  $sql_end = qpg($fb_end);
+  $params[':range_end'] = $fb_end;
+  $where .= ' AND rrule_event_overlaps( dtstart, dtend, rrule, :range_start, :range_end) ';
 }
 else {
   $sql_period = str_replace( 'P', '', $fb_period );
@@ -29,14 +31,14 @@ else {
   $sql_period = str_replace( 'H', ' hours ', $sql_period );
   $sql_period = str_replace( 'M', ' minutes ', $sql_period );
   $sql_period = str_replace( 'S', ' seconds ', $sql_period );
-  $sql_end = "( TIMESTAMP " . qpg($fb_start) . " + ( INTERVAL ".qpg($sql_period).") )";
+  $params[':sql_duration'] = $sql_period;
+  $where .= ' AND rrule_event_overlaps( dtstart, dtend, rrule, :range_start::timestamp, :range_start::timestamp + :sql_duration::interval ) ';
 }
 
-$where .= "AND rrule_event_overlaps( dtstart, dtend, rrule, ".qpg($fb_start).", $sql_end ) ";
 $where .= "AND caldav_data.caldav_type IN ( 'VEVENT', 'VFREEBUSY' ) ";
 $where .= "AND (calendar_item.transp != 'TRANSPARENT' OR calendar_item.transp IS NULL) ";
 $where .= "AND (calendar_item.status != 'CANCELLED' OR calendar_item.status IS NULL) ";
-if ( ! $request->AllowedTo('all') ) {
+if ( $request->Privileges() != privilege_to_bits('all') ) {
   $where .= "AND (calendar_item.class != 'PRIVATE' OR calendar_item.class IS NULL) ";
 }
 
@@ -48,13 +50,13 @@ $sql .= "to_char(calendar_item.dtend at time zone 'GMT',".iCalendar::SqlUTCForma
 $sql .= "FROM caldav_data INNER JOIN calendar_item USING(dav_id,user_no,dav_name)".$where;
 if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY dav_id";
 // echo $sql. "\n";
-$qry = new PgQuery( $sql );
-if ( $qry->Exec("freebusy",__LINE__,__FILE__) && $qry->rows > 0 ) {
+$qry = new AwlQuery( $sql, $params );
+if ( $qry->Exec("freebusy",__LINE__,__FILE__) && $qry->rows() > 0 ) {
   while( $calendar_object = $qry->Fetch() ) {
     if ( $calendar_object->transp != "TRANSPARENT" ) {
       switch ( $calendar_object->status ) {
         case "TENTATIVE":
-          dbg_error_log( "REPORT", " FreeBusy: tentative appointment: %s, %s", $calendar_object->start, $calendar_object->finish );
+          dbg_error_log( "freebusy", " FreeBusy: tentative appointment: %s, %s", $calendar_object->start, $calendar_object->finish );
           $busy_tentative[] = $calendar_object;
           break;
 
@@ -63,15 +65,24 @@ if ( $qry->Exec("freebusy",__LINE__,__FILE__) && $qry->rows > 0 ) {
           break;
 
         default:
-          dbg_error_log( "REPORT", " FreeBusy: Not transparent, tentative or cancelled: %s, %s", $calendar_object->start, $calendar_object->finish );
+          dbg_error_log( "freebusy", " FreeBusy: Not transparent, tentative or cancelled: %s, %s", $calendar_object->start, $calendar_object->finish );
           $busy[] = $calendar_object;
           break;
       }
     }
   }
 }
-$freebusy = iCalendar::iCalHeader();
-$freebusy .= sprintf("BEGIN:VFREEBUSY\nDTSTAMP:%s\nDTSTART:%s\n%s:%s\n", date('Ymd\THis\Z'), $begindate->RenderGMT(), (isset($fb_period) ? "DURATION" : "DTEND"), (isset($fb_period) ? $fb_period : $enddate->RenderGMT()));
+
+$freebusy = new iCalComponent();
+$freebusy->SetType('VFREEBUSY');
+$freebusy->AddProperty('DTSTAMP', date('Ymd\THis\Z'));
+$freebusy->AddProperty('DTSTART', $begindate->RenderGMT());
+if ( isset($fb_period) ) {
+  $freebusy->AddProperty('DURATION', $fb_period);
+}
+else {
+  $freebusy->AddProperty('DTEND', $enddate->RenderGMT());
+}
 
 foreach( $busy_tentative AS $k => $v ) {
   $start = new iCalDate($v->start);
@@ -84,11 +95,11 @@ foreach( $busy_tentative AS $k => $v ) {
       if ( $date->GreaterThan($enddate) ) break;
       $todate = clone($date);
       $todate->AddDuration($duration);
-      $freebusy .= sprintf("FREEBUSY;FBTYPE=BUSY-TENTATIVE:%s/%s\n", $date->RenderGMT(), $todate->RenderGMT() );
+      $freebusy->AddProperty( 'FREEBUSY', $date->RenderGMT().'/'.$todate->RenderGMT(), array('FBTYPE' => 'BUSY-TENTATIVE') );
     }
   }
   else {
-    $freebusy .= sprintf("FREEBUSY;FBTYPE=BUSY-TENTATIVE:%s/%s\n", $start->RenderGMT(), $finish->RenderGMT() );
+    $freebusy->AddProperty( 'FREEBUSY', $start->RenderGMT().'/'.$finish->RenderGMT(), array('FBTYPE' => 'BUSY-TENTATIVE') );
   }
 }
 
@@ -104,15 +115,17 @@ foreach( $busy AS $k => $v ) {
       if ( $date->GreaterThan($enddate) ) break;
       $todate = clone($date);
       $todate->AddDuration($duration);
-      $freebusy .= sprintf("FREEBUSY:%s/%s\n", $date->RenderGMT(), $todate->RenderGMT() );
+      $freebusy->AddProperty( 'FREEBUSY', $date->RenderGMT().'/'.$todate->RenderGMT() );
     }
   }
   else {
-    $freebusy .= sprintf("FREEBUSY:%s/%s\n", $start->RenderGMT(), $finish->RenderGMT() );
+    $freebusy->AddProperty( 'FREEBUSY', $start->RenderGMT().'/'.$finish->RenderGMT() );
   }
 }
 
-$freebusy .= "END:VFREEBUSY\n";
-$freebusy .= iCalendar::iCalFooter();
-$request->DoResponse( 200, $freebusy, 'text/calendar' );
+$result = new iCalComponent();
+$result->VCalendar();
+$result->AddComponent($freebusy);
+
+$request->DoResponse( 200, $result->Render(), 'text/calendar' );
 // Won't return from that
