@@ -6,6 +6,9 @@ DBNAME=regression
 PGPOOL=inactive
 HOSTNAME=regression
 
+# We need to run the regression tests in the timezone they were written for.
+export PGTZ=Pacific/Auckland
+
 . ./regression.conf
 
 [ -z "${DSN}" ] && DSN="${DBNAME}"
@@ -16,7 +19,8 @@ HOSTNAME=regression
 UNTIL=${1:-"99999"}
 ACCEPT_ALL=${2:-""}
 
-REGRESSION="tests/regression-suite"
+[ -z "${SUITE}" ] && SUITE="regression-suite"
+REGRESSION="tests/${SUITE}"
 RESULTS="${REGRESSION}/results"
 
 check_result() {
@@ -24,7 +28,7 @@ check_result() {
   if [ ! -f "${REGRESSION}/${TEST}.result" ] ; then
     touch "${REGRESSION}/${TEST}.result"
   fi
-  diff -u "${REGRESSION}/${TEST}.result" "${RESULTS}/${TEST}" >"${REGRESSION}/diffs/${TEST}"
+  diff --text -u "${REGRESSION}/${TEST}.result" "${RESULTS}/${TEST}" >"${REGRESSION}/diffs/${TEST}"
 
   if [ -s "${REGRESSION}/diffs/${TEST}" ] ; then
     echo "======================================="
@@ -33,20 +37,22 @@ check_result() {
     cat "${REGRESSION}/diffs/${TEST}"
     echo "======================================="
     if [ "${ACCEPT_ALL}" = "" ] ; then
-      read -p "[${TEST}] Accept new result [e/r/v/x/y/N]? " ACCEPT
+      read -p "[${TEST}] Accept new result [e/r/v/f/x/y/N]? " ACCEPT
     else
       ACCEPT=${ACCEPT_ALL}
     fi
     if [ "${ACCEPT}" = "y" ] ; then
       cp "${RESULTS}/${TEST}" "${REGRESSION}/${TEST}.result"
     elif [ "${ACCEPT}" = "x" ]; then
-      echo "./dav_test --dsn '${DSN}' ${WEBHOST} ${ALTHOST} --suite regression-suite --case '${TEST}' --debug"
+      echo "./dav_test --dsn '${DSN}' ${WEBHOST} ${ALTHOST} --suite '${SUITE}' --case '${TEST}' --debug"
       exit
     elif [ "${ACCEPT}" = "v" ]; then
       echo "Showing test $REGRESSION/${TEST}.test"
       cat "$REGRESSION/${TEST}.test"
       return 2
     elif [ "${ACCEPT}" = "f" ]; then
+      echo "Showing full details of ${TEST}"
+      cat "${REGRESSION}/${TEST}.test"
       echo "Showing full result of ${TEST}"
       cat "${RESULTS}/${TEST}"
       return 2
@@ -109,7 +115,7 @@ for T in ${REGRESSION}/*.test ; do
 
   RESULT=999
   while [ "${RESULT}" -gt 0 ]; do
-    ./dav_test --dsn "${DSN}" ${WEBHOST} ${ALTHOST} --suite regression-suite --case "${TEST}" | ./normalise_result > "${RESULTS}/${TEST}"
+    ./dav_test --dsn "${DSN}" ${WEBHOST} ${ALTHOST} --suite "${SUITE}" --case "${TEST}" | ./normalise_result > "${RESULTS}/${TEST}"
 
     RESULT=999
     while [ "${RESULT}" -gt 1 ]; do
